@@ -2,12 +2,14 @@ require_relative '../../spec_helper'
 require_relative 'fixtures/classes'
 
 describe "Kernel#untrusted?" do
-  ruby_version_is ""..."3.0" do
+  ruby_version_is ""..."3.2" do
     it "is a no-op" do
-      o = mock('o')
-      o.should_not.untrusted?
-      o.untrust
-      o.should_not.untrusted?
+      suppress_warning do
+        o = mock('o')
+        o.should_not.untrusted?
+        o.untrust
+        o.should_not.untrusted?
+      end
     end
 
     it "warns in verbose mode" do
@@ -17,4 +19,10 @@ describe "Kernel#untrusted?" do
       }.should complain(/Object#untrusted\? is deprecated and will be removed in Ruby 3.2/, verbose: true)
     end
   end
+
+  ruby_version_is "3.2" do
+    it "has been removed" do
+      Object.new.should_not.respond_to?(:untrusted?)
+    end
+  end
 end
