@@ -15,6 +15,7 @@ class TestIO_Console < Test::Unit::TestCase
     raise
   end
   PATHS.uniq!
+  INCLUDE_OPTS = "-I#{PATHS.join(File::PATH_SEPARATOR)}"
 
   # FreeBSD seems to hang on TTOU when running parallel tests
   # tested on FreeBSD 11.x.
@@ -36,22 +37,22 @@ class TestIO_Console < Test::Unit::TestCase
     trap(:TTOU, @old_ttou) if defined?(@old_ttou) and @old_ttou
   end
 
+  exceptions = %w[ENODEV ENOTTY EBADF ENXIO].map {|e|
+    Errno.const_get(e) if Errno.const_defined?(e)
+  }
+  exceptions.compact!
+  FailedPathExceptions = (exceptions unless exceptions.empty?)
+
   def test_failed_path
-    exceptions = %w[ENODEV ENOTTY EBADF ENXIO].map {|e|
-      Errno.const_get(e) if Errno.const_defined?(e)
-    }
-    exceptions.compact!
-    omit if exceptions.empty?
     File.open(IO::NULL) do |f|
-      e = assert_raise(*exceptions) do
+      e = assert_raise(*FailedPathExceptions) do
         f.echo?
       end
       assert_include(e.message, IO::NULL)
     end
-  end
+  end if FailedPathExceptions
 
   def test_bad_keyword
-    omit if RUBY_ENGINE == 'jruby'
     assert_raise_with_message(ArgumentError, /unknown keyword:.*bad/) do
       File.open(IO::NULL) do |f|
         f.raw(bad: 0)
@@ -440,6 +441,11 @@ defined?(PTY) and defined?(IO.console) and TestIO_Console.class_eval do
     def test_sync
       assert_equal(["true"], run_pty("p IO.console.sync"))
     end
+
+    def test_ttyname
+      return unless IO.method_defined?(:ttyname)
+      assert_equal(["true"], run_pty("p STDIN.ttyname == STDOUT.ttyname"))
+    end
   end
 
   private
@@ -457,7 +463,7 @@ defined?(PTY) and defined?(IO.console) and TestIO_Console.class_eval do
   def run_pty(src, n = 1)
     pend("PTY.spawn cannot control terminal on JRuby") if RUBY_ENGINE == 'jruby'
 
-    args = ["-I#{TestIO_Console::PATHS.join(File::PATH_SEPARATOR)}", "-rio/console", "-e", src]
+    args = [TestIO_Console::INCLUDE_OPTS, "-rio/console", "-e", src]
     args.shift if args.first == "-I" # statically linked
     r, w, pid = PTY.spawn(EnvUtil.rubybin, *args)
   rescue RuntimeError
@@ -530,6 +536,13 @@ defined?(IO.console) and TestIO_Console.class_eval do
     def test_getch_timeout
       assert_nil(IO.console.getch(intr: true, time: 0.1, min: 0))
     end
+
+    def test_ttyname
+      return unless IO.method_defined?(:ttyname)
+      ttyname = IO.console.ttyname
+      assert_not_nil(ttyname)
+      File.open(ttyname) {|f| assert_predicate(f, :tty?)}
+    end
   end
 end
 
@@ -545,17 +558,18 @@ defined?(IO.console) and TestIO_Console.class_eval do
   if noctty
     require 'tempfile'
     NOCTTY = noctty
-    def test_noctty
+    def run_noctty(src)
       t = Tempfile.new("noctty_out")
       t.close
       t2 = Tempfile.new("noctty_run")
       t2.close
       cmd = [*NOCTTY[1..-1],
+        TestIO_Console::INCLUDE_OPTS,
         '-e', 'open(ARGV[0], "w") {|f|',
         '-e',   'STDOUT.reopen(f)',
         '-e',   'STDERR.reopen(f)',
         '-e',   'require "io/console"',
-        '-e',   'f.puts IO.console.inspect',
+        '-e',   "f.puts (#{src}).inspect",
         '-e',   'f.flush',
         '-e',   'File.unlink(ARGV[1])',
         '-e', '}',
@@ -566,11 +580,18 @@ defined?(IO.console) and TestIO_Console.class_eval do
         sleep 0.1
       end
       t.open
-      assert_equal("nil", t.gets(nil).chomp)
+      t.gets.lines(chomp: true)
     ensure
       t.close! if t and !t.closed?
       t2.close!
     end
+
+    def test_noctty
+      assert_equal(["nil"], run_noctty("IO.console"))
+      if IO.method_defined?(:ttyname)
+        assert_equal(["nil"], run_noctty("STDIN.ttyname rescue $!"))
+      end
+    end
   end
 end
 
