@@ -57,20 +57,19 @@ class TestInteger < Test::Unit::TestCase
                           nil
                         end, "[ruby-dev:32084] [ruby-dev:34547]")
 
-    x = EnvUtil.suppress_warning {2 ** -0x4000000000000000}
-    assert_in_delta(0.0, (x / 2), Float::EPSILON)
+    assert_raise(ArgumentError) {2 ** -0x4000000000000000}
 
     <<~EXPRS.each_line.with_index(__LINE__+1) do |expr, line|
       crash01: 111r+11**-11111161111111
       crash02: 1118111111111**-1111111111111111**1+1==11111
-      crash03: -1111111**-1111*11 - -1111111** -111111111
+      crash03: -1111111**-1111*11 - -11** -1111111
       crash04: 1118111111111** -1111111111111111**1+11111111111**1 ===111
       crash05: 11** -111155555555555555  -55   !=5-555
       crash07: 1 + 111111111**-1111811111
       crash08: 18111111111**-1111111111111111**1 + 1111111111**-1111**1
       crash10: -7 - -1111111** -1111**11
       crash12: 1118111111111** -1111111111111111**1 + 1111 - -1111111** -1111*111111111119
-      crash13: 1.0i - -1111111** -111111111
+      crash13: 1.0i - -11** -1111111
       crash14: 11111**111111111**111111 * -11111111111111111111**-111111111111
       crash15: ~1**1111 + -~1**~1**111
       crash17: 11** -1111111**1111 /11i
@@ -80,7 +79,7 @@ class TestInteger < Test::Unit::TestCase
       crash21: 11**-10111111119-1i -1r
     EXPRS
       name, expr = expr.split(':', 2)
-      assert_ruby_status(%w"-W0", expr, name)
+      assert_ruby_status(%w"-W0", "begin; #{ expr }; rescue ArgumentError; end", name)
     end
   end
 
@@ -138,20 +137,6 @@ class TestInteger < Test::Unit::TestCase
     assert_equal(1234, Integer(1234))
     assert_equal(1, Integer(1.234))
 
-    # base argument
-    assert_equal(1234, Integer("1234", 10))
-    assert_equal(668, Integer("1234", 8))
-    assert_equal(4660, Integer("1234", 16))
-    assert_equal(49360, Integer("1234", 36))
-    # decimal, not octal
-    assert_equal(1234, Integer("01234", 10))
-    assert_raise(ArgumentError) { Integer("0x123", 10) }
-    assert_raise(ArgumentError) { Integer(1234, 10) }
-    assert_raise(ArgumentError) { Integer(12.34, 10) }
-    assert_raise(ArgumentError) { Integer(Object.new, 1) }
-
-    assert_raise(ArgumentError) { Integer(1, 1, 1) }
-
     assert_equal(2 ** 50, Integer(2.0 ** 50))
     assert_raise(TypeError) { Integer(nil) }
 
@@ -252,6 +237,32 @@ class TestInteger < Test::Unit::TestCase
     assert_equal(16, Integer(obj))
   end
 
+  def test_Integer_with_base
+    assert_equal(1234, Integer("1234", 10))
+    assert_equal(668, Integer("1234", 8))
+    assert_equal(4660, Integer("1234", 16))
+    assert_equal(49360, Integer("1234", 36))
+    # decimal, not octal
+    assert_equal(1234, Integer("01234", 10))
+    assert_raise(ArgumentError) { Integer("0x123", 10) }
+    assert_raise(ArgumentError) { Integer(1234, 10) }
+    assert_raise(ArgumentError) { Integer(12.34, 10) }
+    assert_raise(ArgumentError) { Integer(Object.new, 1) }
+
+    assert_raise(ArgumentError) { Integer(1, 1, 1) }
+
+    def (base = Object.new).to_int
+      8
+    end
+    assert_equal(8, Integer("10", base))
+
+    assert_raise(TypeError) { Integer("10", "8") }
+    def (base = Object.new).to_int
+      "8"
+    end
+    assert_raise(TypeError) { Integer("10", base) }
+  end
+
   def test_int_p
     assert_not_predicate(1.0, :integer?)
     assert_predicate(1, :integer?)
@@ -270,31 +281,31 @@ class TestInteger < Test::Unit::TestCase
 
   def test_upto
     a = []
-    1.upto(3) {|x| a << x }
+    assert_equal(1, 1.upto(3) {|x| a << x })
     assert_equal([1, 2, 3], a)
 
     a = []
-    1.upto(0) {|x| a << x }
+    assert_equal(1, 1.upto(0) {|x| a << x })
     assert_equal([], a)
 
     y = 2**30 - 1
     a = []
-    y.upto(y+2) {|x| a << x }
+    assert_equal(y, y.upto(y+2) {|x| a << x })
     assert_equal([y, y+1, y+2], a)
   end
 
   def test_downto
     a = []
-    -1.downto(-3) {|x| a << x }
+    assert_equal(-1, -1.downto(-3) {|x| a << x })
     assert_equal([-1, -2, -3], a)
 
     a = []
-    1.downto(2) {|x| a << x }
+    assert_equal(1, 1.downto(2) {|x| a << x })
     assert_equal([], a)
 
     y = -(2**30)
     a = []
-    y.downto(y-2) {|x| a << x }
+    assert_equal(y, y.downto(y-2) {|x| a << x })
     assert_equal([y, y-1, y-2], a)
   end
 
@@ -309,23 +320,34 @@ class TestInteger < Test::Unit::TestCase
     begin;
       called = false
       Integer.class_eval do
-        alias old_plus +
-        undef +
-        define_method(:+){|x| called = true; 1}
+        alias old_succ succ
+        undef succ
+        define_method(:succ){|x| called = true; x+1}
         alias old_lt <
         undef <
         define_method(:<){|x| called = true}
       end
+
+      fix = 1
+      fix.times{break 0}
+      fix_called = called
+
+      called = false
+
       big = 2**65
       big.times{break 0}
+      big_called = called
+
       Integer.class_eval do
-        undef +
-        alias + old_plus
+        undef succ
+        alias succ old_succ
         undef <
         alias < old_lt
       end
+
+      # Asssert that Fixnum and Bignum behave consistently
       bug18377 = "[ruby-core:106361]"
-      assert_equal(false, called, bug18377)
+      assert_equal(fix_called, big_called, bug18377)
     end;
   end
 
@@ -689,6 +711,14 @@ class TestInteger < Test::Unit::TestCase
   def test_fdiv
     assert_equal(1.0, 1.fdiv(1))
     assert_equal(0.5, 1.fdiv(2))
+
+    m = 50 << Float::MANT_DIG
+    prev = 1.0
+    (1..100).each do |i|
+      val = (m + i).fdiv(m)
+      assert_operator val, :>=, prev, "1+epsilon*(#{i}/100)"
+      prev = val
+    end
   end
 
   def test_obj_fdiv
@@ -733,5 +763,9 @@ class TestInteger < Test::Unit::TestCase
 
     assert_equal(10, (10**100-11).ceildiv(10**99-1))
     assert_equal(11, (10**100-9).ceildiv(10**99-1))
+
+    o = Object.new
+    def o.coerce(other); [other, 10]; end
+    assert_equal(124, 1234.ceildiv(o))
   end
 end
