@@ -5,10 +5,10 @@ source "https://rubygems.org"
 gem "rack", "~> 3.0"
 gem "rackup", "~> 2.1"
 gem "base64"
-gem "webrick", "~> 1.8"
+gem "webrick", "~> 1.9"
 gem "rack-test", "~> 2.1"
 gem "compact_index", "~> 0.15.0"
-gem "sinatra", "< 4.1"
+gem "sinatra", "~> 4.1"
 gem "rake", "~> 13.1"
 gem "builder", "~> 3.2"
 gem "rb_sys"
